/*
 *  Gtk-Parted - a front-end for libparted
 *  Copyright (C) 2000, 2001  Damien Genet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "../misc/misc.h"

#include "gped.h"
#include "ctreeParted.h"
#include "ctreeParted_callbacks.h"
#include "popup.h"


/*
 * on right button press event show popup menu
 */
gboolean
on_ctreeParted_button_press_event      (GtkCTree        *ctreeParted,
                                        GdkEventButton  *ev,
                                        gpedInfo        *info)
{	
	g_return_val_if_fail (ctreeParted, FALSE);
	g_return_val_if_fail (GTK_IS_CTREE (ctreeParted), FALSE);
	g_return_val_if_fail (ev, FALSE);
	g_return_val_if_fail (info, FALSE);

	if (ev->button == 3) {
		popup_disable_unavailable_entries (GTK_ITEM_FACTORY (info->item_factory),
		                                   info);
		gtk_item_factory_popup (GTK_ITEM_FACTORY (info->item_factory),
		                        ev->x_root, ev->y_root,
		                        ev->button,
		                        ev->time);
	
		return TRUE;
	}
	
	return FALSE;
}

gboolean
on_ctreeParted_tree_move               (GtkCTree        *ctreeParted,
                                        GtkCTreeNode    *node,
                                        GtkCTreeNode    *new_parent,
                                        GtkCTreeNode    *new_sibling,
                                        gpedInfo	*info)
{
	GtkWidget	*popup;
	PedPartition	*orig_part;
	PedPartition	*free_part;
	PedPartition	*new_part;
	PedFileSystem	*fs;
	PedConstraint	*constraint;

	g_return_val_if_fail (ctreeParted, FALSE);
	g_return_val_if_fail (GTK_IS_CTREE (ctreeParted), FALSE);
	g_return_val_if_fail (node, FALSE);
	g_return_val_if_fail (info, FALSE);
	
	
	/*
	 * test partition validity
	 */
	if ( !(orig_part = gtk_ctree_node_get_row_data (GTK_CTREE (ctreeParted), GTK_CTREE_NODE (node))) )
		g_assert_not_reached ();
	
	if ( orig_part->type != PED_PARTITION_PRIMARY
	     && orig_part->type != PED_PARTITION_LOGICAL )
		goto error_not_good_partition;
		
	/*
	 * test if node has been moved on free space
	 */
	if ( !new_parent || !new_sibling )
		goto error_not_on_free_space;
	
	if ( !(free_part = gtk_ctree_node_get_row_data (GTK_CTREE (ctreeParted), GTK_CTREE_NODE (new_sibling))) )
		g_assert_not_reached ();
	
	if ( free_part->type != PED_PARTITION_FREESPACE )
		goto error_not_on_free_space;
	
	if ( free_part->geom.length < orig_part->geom.length )
		goto error_not_enough_free_space;

	/*
	 * copy fs
	 */
	if ( gped_get_changes (info) ) {
		dialog_option	ret;
		
		ret = ui_dialog_opt_create_and_run (_("Changes must be written before copying Partition !"),
		                                    DIALOG_OK_CANCEL);
		
		if (ret == DIALOG_CANCEL)
			goto return_ok;
	}

	popup = ui_dialogPopup_create (_("Copying FileSystem,\n please wait..."));
	gtk_thread_inc_ref_count ();

	if ( !(fs = ped_file_system_open (&orig_part->geom)) )
		goto error_destroy_popup;

	if ( !(new_part = ped_partition_new (free_part->geom.disk,
	                                     free_part->type & ~PED_PARTITION_FREESPACE,
	                                     fs->type,
	                                     free_part->geom.start,
	                                     free_part->geom.start + orig_part->geom.length-1)) )
		goto error_close_fs;

	constraint = ped_constraint_any (new_part->geom.disk);
	if ( !ped_disk_add_partition (new_part->geom.disk, new_part, constraint) )
		goto error_free_constraint;
	ped_constraint_destroy (constraint);
	
	ped_file_system_copy (fs, &new_part->geom);
	
	ped_file_system_close (fs);
	
	gtk_thread_dec_ref_count ();
	gtk_widget_destroy (GTK_WIDGET (popup));
	ctreeParted_refresh (GTK_CTREE (ctreeParted), info);
	
return_ok:
	gtk_signal_emit_stop_by_name (GTK_OBJECT (ctreeParted),
	                              "tree_move");
	return TRUE;

/*
 * error returns
 */ 
error_not_good_partition:
	ui_dialog_opt_create_and_run (_("This is not a primary or logical partition"),
	                              DIALOG_CANCEL);
	goto error;

error_not_on_free_space:
	ui_dialog_opt_create_and_run (_("FileSystem can only be copied on FreeSpace"),
	                              DIALOG_CANCEL);
	goto error;

error_not_enough_free_space:
	ui_dialog_opt_create_and_run (_("Available FreeSpace is too small"),
	                              DIALOG_CANCEL);
	goto error;

error_free_constraint:
	ped_constraint_destroy (constraint);
	ped_partition_destroy (new_part);

error_close_fs:
	ped_file_system_close (fs);

error_destroy_popup:
	gtk_thread_dec_ref_count ();
	gtk_widget_destroy (GTK_WIDGET (popup));

error:
	gtk_signal_emit_stop_by_name (GTK_OBJECT (ctreeParted),
	                              "tree_move");
	return FALSE;
}
