/*
 *  Gtk-Parted - a front-end for libparted
 *  Copyright (C) 2000, 2001  Damien Genet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * GUI functions
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include <parted/parted.h>
#include <gtk/gtk.h>

#include "../misc/misc.h"

#include "gped.h"
#include "fstab.h"
#include "dialogOp.h"
#include "dialogOp_private.h"
#include "dialogOp_callbacks.h"


static GtkWidget*
dialogOp_create                        (gpedOpInfo      *op,
                                        gpedInfo        *info,
                                        PedPartition    *part,
                                        gchar           *title)
{
	GtkWidget	*dialogOp;

	g_return_val_if_fail (op, NULL);
	g_return_val_if_fail (info, NULL);
	g_return_val_if_fail (part, NULL);
	g_return_val_if_fail (title,NULL);
	
	op->info = info;
	op->part = part;
	op->disk = part->geom.disk;
	
	dialogOp = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (dialogOp), title);
	gtk_window_set_modal (GTK_WINDOW (dialogOp), TRUE);

	gtk_signal_connect (GTK_OBJECT (dialogOp), "delete_event",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    NULL);
	
	return dialogOp;
}

static GtkWidget*
dialogOp_add_table                    (GtkWindow       *dialogOp)
{
	GtkWidget	*table;

	g_return_val_if_fail (dialogOp, NULL);
	g_return_val_if_fail (GTK_IS_WINDOW (dialogOp), NULL);
	
	table = gtk_table_new (6, 4, FALSE);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogOp)->vbox), table, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (table), 10);
	gtk_table_set_row_spacings (GTK_TABLE (table), 8);
	gtk_table_set_col_spacings (GTK_TABLE (table), 4);

	return table;
}


static GList*
listFSType_get                        (void)
{
	PedFileSystemType	*fs_type = NULL;
	static GList		*listFSType = NULL;
	
	if (listFSType)
		return listFSType;
	
	while ( (fs_type = ped_file_system_type_get_next (fs_type)) )
		listFSType = g_list_append (listFSType, (gchar*) fs_type->name);
		
	return listFSType;
}

static void
dialogOp_add_system_type              (gpedOpInfo         *op,
                                       GtkTable           *table)
{
	GtkWidget	*labelSystemType;
	GtkWidget	*comboSystemType;
	GList		*listFSType;

	g_return_if_fail (op);
	g_return_if_fail (op->part);
	g_return_if_fail (table);
	g_return_if_fail (GTK_IS_TABLE (table));

	labelSystemType = gtk_label_new (_("Partition Type"));
	gtk_table_attach (GTK_TABLE (table), labelSystemType, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	gtk_label_set_justify (GTK_LABEL (labelSystemType), GTK_JUSTIFY_RIGHT);
	
	comboSystemType = gtk_combo_new ();
	gtk_table_attach (GTK_TABLE (table), comboSystemType, 1, 4, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	listFSType = listFSType_get ();
	gtk_combo_set_popdown_strings (GTK_COMBO (comboSystemType), listFSType);
	gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (comboSystemType)->entry),
	                    op->part->fs_type ? op->part->fs_type->name : N_("unknown"));
	op->comboSystemType = GTK_COMBO (comboSystemType);

	gtk_signal_connect (GTK_OBJECT (GTK_COMBO (comboSystemType)->entry), "changed",
	                    GTK_SIGNAL_FUNC (on_comboSystemType_changed),
	                    op);
}


static void
_add_flag                             (gpedOpInfo      *op,
                                       GtkTable        *tableFlags,
                                       PedPartitionFlag flag,
                                       gushort          x_pos,
                                       gushort          y_pos)
{
	GtkWidget	*checkbuttonFlag;
	const gchar	*name;
	
	g_return_if_fail (tableFlags);
	g_return_if_fail (GTK_IS_TABLE (tableFlags));
	g_return_if_fail (op);
	g_return_if_fail (op->part);
	g_return_if_fail (flag);
	g_return_if_fail (flag <= PED_PARTITION_LAST_FLAG);
	
	name = ped_partition_flag_get_name (flag);
	checkbuttonFlag = gtk_check_button_new_with_label (name);
	gtk_table_attach (GTK_TABLE (tableFlags), checkbuttonFlag, x_pos, x_pos+1, y_pos, y_pos+1,
	                  (GtkAttachOptions) (GTK_EXPAND),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	if ( !ped_partition_is_flag_available (op->part, flag) )
		gtk_widget_set_sensitive (GTK_WIDGET (checkbuttonFlag), FALSE);
	else if ( ped_partition_get_flag (op->part, flag) )
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbuttonFlag), TRUE);

	op->checkbuttonFlags[flag-1] = GTK_CHECK_BUTTON (checkbuttonFlag);
}


static void
dialogOp_add_flags                    (gpedOpInfo       *op,
                                       GtkTable         *table)
{
	GtkWidget		*tableFlags;
	PedPartitionFlag	 flag = 0;
	gushort			 x_pos = 0;
	gushort			 y_pos = 0;
	
	g_return_if_fail (table);
	g_return_if_fail (GTK_IS_TABLE (table));
	g_return_if_fail (op);
	g_return_if_fail (op->part);

	tableFlags = gtk_table_new (2 , 4, TRUE);
	gtk_table_attach (GTK_TABLE (table), tableFlags, 0, 4, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);

	while ( (flag = ped_partition_flag_next (flag)) ) {
		_add_flag (op, GTK_TABLE (tableFlags), flag,
		           x_pos, y_pos);
		y_pos = (x_pos == 3) ? y_pos+1 : y_pos;
		x_pos = (x_pos == 3) ? 0 : x_pos+1;
	}
}



static void
dialogOp_add_size_spin_buttons         (gpedOpInfo      *op,
                                        GtkTable        *table,
                                        PedGeometry     *max_geom)
{
	GtkObject	*spinbuttonSize_adj;
	GtkWidget	*spinbuttonSize;
	GtkObject	*spinbuttonStart_adj;
	GtkWidget	*spinbuttonStart;
	GtkObject	*spinbuttonEnd_adj;
	GtkWidget	*spinbuttonEnd;
	PedGeometry	*geom;
	
	g_return_if_fail (op);
	g_return_if_fail (table);
	g_return_if_fail (GTK_IS_TABLE (table));
	
	geom = &op->part->geom;
	
	spinbuttonSize_adj = gtk_adjustment_new (geom->length * geom->disk->dev->sector_size / one_MB,
	                                         0,
	                                         max_geom->length * geom->disk->dev->sector_size / one_MB,
	                                         1,
	                                         10, 10);
	spinbuttonSize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonSize_adj), 1, 0);
	gtk_table_attach (GTK_TABLE (table), spinbuttonSize, 1, 4, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	
	spinbuttonStart_adj = gtk_adjustment_new (geom->start,
	                                          max_geom->start,
	                                          max_geom->end,
	                                          1, 10, 10);
	spinbuttonStart = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonStart_adj), 1, 0);
	gtk_table_attach (GTK_TABLE (table), spinbuttonStart, 1, 2, 3, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	
	spinbuttonEnd_adj = gtk_adjustment_new (geom->end,
	                                        max_geom->start,
	                                        max_geom->end,
	                                        1, 10, 10);
	spinbuttonEnd = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonEnd_adj), 1, 0);
	gtk_table_attach (GTK_TABLE (table), spinbuttonEnd, 3, 4, 3, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	
	op->spinbuttonSize = GTK_SPIN_BUTTON (spinbuttonSize);
	op->spinbuttonStart = GTK_SPIN_BUTTON (spinbuttonStart);
	op->spinbuttonEnd = GTK_SPIN_BUTTON (spinbuttonEnd);

	gtk_signal_connect (GTK_OBJECT (spinbuttonSize_adj), "value_changed",
	                    GTK_SIGNAL_FUNC (on_spinbuttonSize_adj_value_changed),
	                    op);
	gtk_signal_connect (GTK_OBJECT (spinbuttonStart_adj), "value_changed",
	                    GTK_SIGNAL_FUNC (on_spinbuttonStart_adj_value_changed),
	                    op);
	gtk_signal_connect (GTK_OBJECT (spinbuttonEnd_adj), "value_changed",
	                    GTK_SIGNAL_FUNC (on_spinbuttonEnd_adj_value_changed),
	                    op);
}

static void
dialogOp_add_size_labels               (gpedOpInfo       *op,
                                        GtkTable         *table)
{
	GtkWidget	*labelStart;
	GtkWidget	*labelEnd;
	GtkWidget	*labelSize;
	
	g_return_if_fail (op);
	g_return_if_fail (table);
	g_return_if_fail (GTK_IS_TABLE (table));
	
	labelSize = gtk_label_new (_("Size (MB)"));
	gtk_table_attach (GTK_TABLE (table), labelSize, 0, 1, 2, 3,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);

	labelStart = gtk_label_new (_("Start"));
	gtk_table_attach (GTK_TABLE (table), labelStart, 0, 1, 3, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (labelStart), 0.9, 0.5);

	labelEnd = gtk_label_new (_("End"));
	gtk_table_attach (GTK_TABLE (table), labelEnd, 2, 3, 3, 4,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (labelEnd), 0.9, 0.5);

	op->labelStart = GTK_LABEL (labelStart);
}

static void
dialogOp_add_size                      (gpedOpInfo      *op,
                                        GtkTable        *table,
                                        PedGeometry     *max_geom)
{
	g_return_if_fail (op);
	g_return_if_fail (table);
	g_return_if_fail (GTK_IS_TABLE (table));
	g_return_if_fail (max_geom);
	
	dialogOp_add_size_labels (op, GTK_TABLE (table));
	dialogOp_add_size_spin_buttons (op,
	                                GTK_TABLE (table),
	                                max_geom);
}

static void
dialogOp_add_mount_point               (gpedOpInfo      *op,
                                        GtkTable        *table,
                                        gchar		*mountpoint)
{
	GtkWidget	*labelMount;
	GtkWidget	*entryMount;

	g_return_if_fail (op);
	g_return_if_fail (table);
	g_return_if_fail (GTK_IS_TABLE (table));

	labelMount = gtk_label_new (_("Mount Point"));
	gtk_table_attach (GTK_TABLE (table), labelMount, 0, 1, 5, 6,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (labelMount), 0.9, 0.5);
	op->labelMount = GTK_LABEL (labelMount);

	entryMount = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE (table), entryMount, 1, 4, 5, 6,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	if (mountpoint)
		gtk_entry_set_text (GTK_ENTRY (entryMount), mountpoint);

	op->entryMount = GTK_ENTRY (entryMount);
}

static void
dialogOp_add_buttons                  (GtkWindow        *dialogOp,
                                       GtkSignalFunc	 ok_callback,
                                       gpointer         *ok_data)
{
	GtkWidget	*hbuttonbox;
	GtkWidget	*buttonOk;
	GtkWidget	*buttonCancel;

	hbuttonbox = gtk_hbutton_box_new ();
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialogOp)->action_area), hbuttonbox, TRUE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);

	buttonOk = gtk_button_new_with_label (_("OK"));
	gtk_container_add (GTK_CONTAINER (hbuttonbox), buttonOk);
	gtk_button_set_relief (GTK_BUTTON (buttonOk), GTK_RELIEF_NONE);
	GTK_WIDGET_SET_FLAGS (buttonOk, GTK_CAN_DEFAULT);

	buttonCancel = gtk_button_new_with_label (_("Cancel"));
	gtk_container_add (GTK_CONTAINER (hbuttonbox), buttonCancel);
	gtk_button_set_relief (GTK_BUTTON (buttonCancel), GTK_RELIEF_NONE);
	GTK_WIDGET_SET_FLAGS (buttonCancel, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (buttonOk), "clicked",
	                    GTK_SIGNAL_FUNC (ok_callback),
	                    ok_data);
	gtk_signal_connect_object_after (GTK_OBJECT (buttonOk), "clicked",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    GTK_OBJECT (dialogOp));
	gtk_signal_connect_object (GTK_OBJECT (buttonCancel), "clicked",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    GTK_OBJECT (dialogOp));

	gtk_widget_grab_default (buttonCancel);
}

static void
dialogProperties_add_resize_file_system (gpedPropInfo       *prop,
                                         GtkTable           *table)
{
	GtkWidget	*checkbuttonResizeFS;
	gchar		*buffer;
	
	g_return_if_fail (prop);
	g_return_if_fail (GPED_OP_INFO (prop));
	g_return_if_fail (GPED_OP_INFO (prop)->part);
	g_return_if_fail (table);
	g_return_if_fail (GTK_IS_TABLE (table));
	
	buffer = g_strdup_printf (_("Resize %s FileSystem"), GPED_OP_INFO (prop)->part->fs_type ? GPED_OP_INFO (prop)->part->fs_type->name : _("unknown"));
	checkbuttonResizeFS = gtk_check_button_new_with_label (buffer);
	g_free (buffer);
	gtk_table_attach (GTK_TABLE (table), checkbuttonResizeFS, 0, 4, 4, 5,
	                  (GtkAttachOptions) (GTK_EXPAND),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (checkbuttonResizeFS), 5);
	if (!GPED_OP_INFO (prop)->part->fs_type)
		gtk_widget_set_sensitive (GTK_WIDGET (checkbuttonResizeFS), FALSE);
	prop->checkbuttonResizeCreateFS = GTK_CHECK_BUTTON (checkbuttonResizeFS);

	gtk_signal_connect (GTK_OBJECT (checkbuttonResizeFS), "toggled",
	                    GTK_SIGNAL_FUNC (on_checkbuttonResizeFS_toggled),
	                    prop);
}

static gpedPropInfo*
_gpedPropInfo_new                      (void)
{
	gpedPropInfo	*prop;
	
	if ( !(prop = g_malloc (sizeof (gpedPropInfo))) ) {
		g_printerr ("%s: allocating gpedPropInfo.\n",
		            g_strerror (errno));
		return NULL;
	}
	
	return  prop;
}

GtkWidget*
dialogProperties_create                (PedPartition    *part,
                                        gpedInfo        *info)
{
	GtkWidget	*dialogProperties;
	GtkWidget	*table;
	PedConstraint	*constraint;
	PedGeometry	*max_geom;
	gpedPropInfo	*prop;
	gchar		*buffer;

	g_return_val_if_fail (part, NULL);
	g_return_val_if_fail (info, NULL);

	/*
	 * init PropertiesInfo prop for callback funct
	 */

	if ( !(prop = _gpedPropInfo_new ()) )
		return NULL;

	buffer = g_strdup_printf (_("%s%i properties"), part->geom.disk->dev->path, part->num);
	if ( !(dialogProperties = dialogOp_create (GPED_OP_INFO (prop),
	                                           info, part, buffer)) ) {
		g_free (buffer);
		goto error;
	}
	g_free (buffer);
	prop->saved_fs_type = part->fs_type; 
	
	if ( !(table = dialogOp_add_table (GTK_WINDOW (dialogProperties))) )
		goto error;
	
	dialogOp_add_system_type (GPED_OP_INFO (prop),
	                          GTK_TABLE (table));
	dialogOp_add_flags (GPED_OP_INFO (prop),
	                    GTK_TABLE (table));
	
	constraint = ped_constraint_any (part->geom.disk);
	if ( !(max_geom = ped_disk_get_max_partition_geometry (part->geom.disk, part, constraint)) ) {
			ped_constraint_destroy (constraint);
			goto error;
	}
	ped_constraint_destroy (constraint);
	dialogOp_add_size (GPED_OP_INFO (prop),
	                   GTK_TABLE (table),
	                   max_geom);
	ped_geometry_destroy (max_geom);

	
	dialogProperties_add_resize_file_system (prop, GTK_TABLE (table));

	dialogOp_add_mount_point (GPED_OP_INFO (prop),
	                          GTK_TABLE (table),
	                          fstab_get_part_mount_point (info->fstab_info, part));

	dialogOp_add_buttons (GTK_WINDOW (dialogProperties),
	                      GTK_SIGNAL_FUNC (on_dialogProperties_buttonOk_clicked),
	                      (gpointer) prop);

	if ( part->type & PED_PARTITION_EXTENDED ) {
		gtk_widget_set_sensitive (GTK_WIDGET (GPED_OP_INFO (prop)->labelMount), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET (GPED_OP_INFO (prop)->entryMount), FALSE);
	}	
 
	/*
	 * connect callbacks
	 */
	gtk_signal_connect (GTK_OBJECT (dialogProperties), "destroy_event",
	                    GTK_SIGNAL_FUNC (on_dialogProperties_destroyed),
	                    prop);

	gtk_widget_show_all (dialogProperties);
	return dialogProperties;

error:
	g_free (prop);
/*	ped_partition_destroy (prop->part);*/
	return NULL;
}



static void
dialogAdd_add_create_file_system      (gpedAddInfo        *add,
                                       GtkTable           *table)
{
	GtkWidget	*checkbuttonCreateFS;

	g_return_if_fail (add);
	g_return_if_fail (table);
	g_return_if_fail (GTK_IS_TABLE (table));

	checkbuttonCreateFS = gtk_check_button_new_with_label (_("Create FileSystem"));
	gtk_table_attach (GTK_TABLE (table), checkbuttonCreateFS, 0, 2, 4, 5,
	                  (GtkAttachOptions) (GTK_EXPAND),
	                  (GtkAttachOptions) (GTK_EXPAND), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (checkbuttonCreateFS), 5);
	add->checkbuttonCreateFS = GTK_CHECK_BUTTON (checkbuttonCreateFS);
}

static gpedAddInfo*
_gpedAddInfo_new                       (void)
{
	gpedAddInfo	*add;
	
	if ( !(add = g_malloc (sizeof (gpedAddInfo))) ) {
		g_printerr ("%s: allocating gpedAddInfo.\n",
		            g_strerror (errno));
		return NULL;
	}

	return add;
}

GtkWidget*
dialogAdd_create                       (PedGeometry     *geom,
                                        gpedInfo        *info,
                                        PedPartitionType type)
{
	GtkWidget	*dialogAdd;
	GtkWidget	*table;
	PedPartition	*part;
	gpedAddInfo	*add;

	g_return_val_if_fail (geom, NULL);
	g_return_val_if_fail (info, NULL);

	/*
	 * init PropertiesInfo prop for callback funct
	 */

	if ( !(add = _gpedAddInfo_new ()) )
		return NULL;
	add->type = type;
	
	part = ped_partition_new (geom->disk,
	                          type,
	                          NULL,
	                          geom->start,
	                          geom->end); 

	if ( !(dialogAdd = dialogOp_create (GPED_OP_INFO (add),
	                                    info, part, _("Add Partition"))) )
		goto error;
	
	if ( !(table = dialogOp_add_table (GTK_WINDOW (dialogAdd))) )
		goto error;
	
	dialogOp_add_system_type (GPED_OP_INFO (add),
	                          GTK_TABLE (table));
	dialogOp_add_flags (GPED_OP_INFO (add),
	                    GTK_TABLE (table));
	dialogOp_add_size (GPED_OP_INFO(add),
	                   GTK_TABLE (table),
	                   &part->geom);

	dialogAdd_add_create_file_system (add, GTK_TABLE (table));
	dialogOp_add_mount_point (GPED_OP_INFO (add), GTK_TABLE (table), NULL);
	dialogOp_add_buttons (GTK_WINDOW (dialogAdd),
	                      GTK_SIGNAL_FUNC (on_dialogAdd_buttonOk_clicked),
	                      (gpointer) add);

	if ( type & PED_PARTITION_EXTENDED ) {
		gtk_widget_set_sensitive (GTK_WIDGET (add->checkbuttonCreateFS), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET (GPED_OP_INFO (add)->labelMount), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET (GPED_OP_INFO (add)->entryMount), FALSE);
	}	


	/*
	 * connect callbacks
	 */
	gtk_signal_connect (GTK_OBJECT (dialogAdd), "destroy_event",
	                    GTK_SIGNAL_FUNC (on_dialogAdd_destroyed),
	                    add);

	gtk_widget_show_all (dialogAdd);
	return dialogAdd;

error:
	ped_partition_destroy (part);
	return NULL;
}
